#!/usr/bin/python
# usage:  ./ValueSetter.py <name of datum> <type of datum> [<list of enumerated types for list datums>]

import curses, curses.wrapper, string, sys, traceback, coredata, os, fnmatch, re

# Constants
ENTER = 10
STANDBY = 50
CHARS = " -" + string.uppercase + string.digits

class ValveState:
    Unknown     = -1
    Off         = 0
    ForwardMin  = 2
    ForwardMax  = 3
    ReverseMin  = 4
    ReverseMax  = 5

class Units:
    Metric = 0
    English = 1

# Global Variables
g_dValveState = 0
g_eValveState = ValveState.Unknown
g_lEnumeratedNames = []
g_eUnits = coredata.int32("Config_UnitSystem").get()

class ValueSetter:
    def __init__(self, name, type):

        # set up a foreground/background color pair (can do many)
        curses.init_pair(1,curses.COLOR_RED,curses.COLOR_BLACK)

        # turn off the cursor
        curses.curs_set(0)

        # Default standby to off
        self.standby = False

        # Maintain our own custom cursor
        self.cursor = 0

        self.length = 7

        if re.search("_kg_km$",name) != None:
            self.conversion = 3.54799619
        elif re.search("_L_Tonne$",name) != None:
            self.conversion = 0.239652855
        elif re.search("_kph$",name) != None:
            self.conversion = 0.621371192
        elif re.search("_kg_rev$",name) != None:
            self.conversion = 2.20462262
        elif re.search("_ml_rev$",name) != None:
            self.conversion = 0.0338140227
        elif re.search("_m$",name) != None:
            self.conversion = 3.2808399
        elif re.search("_pulses_km$",name) != None:
            self.conversion = 1.609344
        else:
            self.conversion = 1

        #Determines the parsing strategy for the ValueSetter
        if type.lower() == "string":
            self.datum = coredata.string(name)
            self.parse = ValueSetter.TextParse
            self.value = []
            i = 0
            self.up    = ""
            self.down  = ""

            # Initialize the displayed value to string datum's value
            while i < self.length:
                if i < len(self.datum.get()):
                    index = CHARS.find(self.datum.get()[i])
                    if index != -1:
                        self.value = [index] + self.value
                    else:
                        self.value = [0] + self.value
                else:
                    self.value = [0] + self.value
                i+=1

        elif type.lower() == "int":
            self.datum = coredata.int32(name)
            self.parse = ValueSetter.NumericParse

            # Setup the value for numbers based on
            if g_eUnits == Units.English:
                self.value = self.conversion * self.datum.get()
            else:
                self.value = self.datum.get()

        elif type.lower() == "timestamped_int":
            self.datum = coredata.timestamped_int32(name)
            self.parse = ValueSetter.NumericParse

            if g_eUnits == Units.English:
                self.value = self.conversion * self.datum.get()
            else:
                self.value = self.datum.get()

        elif type.lower() == "double":
            self.datum = coredata.double(name)
            self.prec = int(sys.argv[5])
            self.parse = ValueSetter.DoubleParse

            self.point = 0

            if g_eUnits == Units.English:
                self.value = self.conversion * self.datum.get()
            else:
                self.value = self.datum.get()

        elif type.lower() == "list":
            self.datum = coredata.int32(name)
            self.parse = ValueSetter.ListParse
            self.cursor = -1
            self.value = self.datum.get()

        elif type.lower() == "bool":
            self.datum = coredata.bool(name)
            self.parse = ValueSetter.BoolParse
            self.value = self.datum.get()
            self.cursor = -1

        elif type.lower() == "upgrade":
            self.parse = ValueSetter.UpgradeParse
            self.value = 0
            self.cursor = -1

        else:
            sys.exit(1)

        self.parse(self,0)

    #Parses a datum of type list
    def UpgradeParse(self,key):
        if key == curses.KEY_UP:
            if self.value + 1 < len(g_lEnumeratedNames):
                self.value += 1
        elif key == curses.KEY_DOWN:
            if self.value > 0:
                self.value -= 1
        elif key == ENTER:
            return False
        elif key == STANDBY:
            self.standby = not self.standby

        self.output = g_lEnumeratedNames[self.value]

        if self.value + 1 < len(g_lEnumeratedNames):
            self.up = g_lEnumeratedNames[self.value + 1]
        else:
            self.up = ""

        if self.value > 0:
            self.down = g_lEnumeratedNames[self.value - 1]
        else:
            self.down = ""

        return True

    #Parses a datum of type list
    def ListParse(self,key):
        if key == curses.KEY_UP:
            if self.value + 1 < len(g_lEnumeratedNames):
                self.value += 1
        elif key == curses.KEY_DOWN:
            if self.value > 0:
                self.value -= 1
        elif key == ENTER:
            return False
        elif key == STANDBY:
            self.standby = not self.standby

        self.output = g_lEnumeratedNames[self.value]
        self.datum.set(self.value)

        if self.value + 1 < len(g_lEnumeratedNames):
            self.up = g_lEnumeratedNames[self.value + 1]
        else:
            self.up = ""

        if self.value > 0:
            self.down = g_lEnumeratedNames[self.value - 1]
        else:
            self.down = ""

        return True

    #Parses a datum of type double
    def DoubleParse(self,key):
        if key == curses.KEY_UP:
            if self.value + 10 ** (self.cursor-self.prec-self.point) < 10 ** (self.length - 1):
                self.value += 10 ** (self.cursor-self.prec-self.point)
        elif key == curses.KEY_DOWN:
            if self.value - 10 ** (self.cursor-self.prec-self.point) > 0:
                self.value -= 10 ** (self.cursor-self.prec-self.point)
            else:
                self.value = 0
        elif key == curses.KEY_RIGHT:
            if self.cursor > 0:
                self.cursor -= 1
            if self.output[len(self.output)-1 - self.cursor] == '.':
                self.point = 0
                self.cursor -= 1
        elif key == curses.KEY_LEFT:
            if self.cursor + 1 < self.length+self.prec:
                self.cursor += 1
            if self.output[len(self.output)-1 - self.cursor] == '.':
                self.point = 1
                self.cursor += 1
        elif key == ENTER:
            return False
        elif key == STANDBY:
            self.standby = not self.standby

        #make it look like an integer if there is no precision
        if self.prec == 0:
            self.output = '%*d' % (self.length,round(self.value))
        else:
            #Format the output to be a number with 'length' zeroes in before the decimal
            #and 'prec' zeroes after the decimal
            self.output = ('%*.*f' % (self.length+self.prec,self.prec,self.value))

        if g_eUnits == Units.English:
            self.datum.set(self.value/self.conversion)
        else:
            self.datum.set(self.value)

        if self.output[-self.cursor-1] == " ":
            self.up = "1"
            self.down = " "
        else:
            if int(self.output[-self.cursor - 1]) + 1 < 10:
                self.up = str(int(self.output[-self.cursor - 1]) + 1)
            else:
                self.up = "0"

            if int(self.output[-self.cursor - 1]) > 0:
                self.down = str(int(self.output[-self.cursor - 1]) - 1)
            else:
                self.down = "9"

        return True

    #Parses a datum of type string
    def TextParse(self,key):
        if key == curses.KEY_UP:
            if self.value[self.cursor] + 1< len(CHARS):
                self.value[self.cursor] += 1
            else:
                self.value[self.cursor] = 0
        elif key == curses.KEY_DOWN:
            if self.value[self.cursor] > 0:
                self.value[self.cursor] -= 1
            else:
                self.value[self.cursor] = len(CHARS) - 1
        elif key == curses.KEY_RIGHT:
            if self.cursor > 0:
                self.cursor -= 1
        elif key == curses.KEY_LEFT:
            if self.cursor + 1 < self.length:
                self.cursor += 1
        elif key == ENTER:
            return False
        elif key == STANDBY:
            self.standby = not self.standby

        self.output = ""
        for cValue in reversed(self.value):
            self.output += CHARS[cValue]
        self.datum.set(self.output)

        if self.value[self.cursor] + 1 < len(CHARS):
            self.up = CHARS[self.value[self.cursor] + 1]
        else:
            self.up = CHARS[0]

        if self.value[self.cursor] > 0:
            self.down = CHARS[self.value[self.cursor] - 1]
        else:
            self.down = CHARS[-1]

        return True

    #Parses a datum of type int32 and timestamped int32
    def NumericParse(self,key):
        if key == curses.KEY_UP:
            if self.value + 10 ** self.cursor < 10 ** self.length:
                self.value += 10 ** self.cursor
        elif key == curses.KEY_DOWN:
            if self.value - 10 ** self.cursor > 0:
                self.value -= 10 ** self.cursor
            else:
                self.value = 0
        elif key == curses.KEY_RIGHT:
            if self.cursor > 0:
                self.cursor -= 1
        elif key == curses.KEY_LEFT:
            if self.cursor + 1 < self.length:
                self.cursor += 1
        elif key == ENTER:
            return False
        elif key == STANDBY:
            self.standby = not self.standby

        self.output = '%*d' % (self.length,self.value)

        if g_eUnits == Units.English:
            self.datum.set(self.value/self.conversion)
        else:
            self.datum.set(self.value)

        if self.output[-self.cursor-1] == " ":
            self.up = "1"
            self.down = " "
        else:
            if int(self.output[-self.cursor - 1]) + 1 < 10:
                self.up = str(int(self.output[-self.cursor - 1]) + 1)
            else:
                self.up = "0"

            if int(self.output[-self.cursor - 1]) > 0:
                self.down = str(int(self.output[-self.cursor - 1]) - 1)
            else:
                self.down = "9"

        return True

    #Parses a datum of type bool
    def BoolParse(self,key):
        if key == curses.KEY_UP:
	    self.value = True
	if key == curses.KEY_DOWN:
            self.value = False
        elif key == ENTER:
            return False
        elif key == STANDBY:
            self.standby = not self.standby

        if self.value:
            self.up = ""
            self.output = "Enabled"
            self.down = "Disabled"
        else:
            self.up = "Enabled"
            self.output = "Disabled"
            self.down = ""

        self.datum.set(self.value)

        return True

def main(stdscr,arg1,arg2,arg3,arg4):

    vs = ValueSetter(arg3,arg4)

    # clear screen
    stdscr.clear()

    # implement the actions done so far (just the clear())
    stdscr.refresh()

    # Main program loop
    while True:

        stdscr.clear()

        # Print the name of the datum and chop of the config if it is there
        stdscr.addstr(0,0,arg1,curses.A_BOLD)

        # If the cursor is valid then print the cursor out
        if vs.cursor != -1:
            stdscr.addch(1,len(vs.output) - 1 - vs.cursor,
                         vs.up,
                         curses.A_DIM)
            stdscr.addstr(2,0,vs.output,curses.A_UNDERLINE|curses.A_BOLD)

            # Print the units if they exist
            if arg2 != "":
                stdscr.addstr(2,len(vs.output)+1,arg2,curses.A_BOLD)

            stdscr.addch(2,len(vs.output) - 1 - vs.cursor,
                         vs.output[-vs.cursor-1],
                         curses.A_BLINK | curses.A_UNDERLINE|curses.A_BOLD)
            stdscr.addch(3,len(vs.output) - 1 - vs.cursor,
                         vs.down,
                         curses.A_DIM)

        # Otherwise just highlight the whole line
        else:
            stdscr.addstr(1,0,vs.up,curses.A_DIM)
            stdscr.addstr(2,0,vs.output,curses.A_UNDERLINE|curses.A_BOLD)
            stdscr.addstr(3,0,vs.down,curses.A_DIM)

        # Handle standby
        if g_eValveState != ValveState.Unknown:
            if vs.standby:
                g_dValveState.set(g_eValveState)
                stdscr.addstr(4,0,"Running Output!",curses.color_pair(1)|curses.A_BOLD)
            else:
                g_dValveState.set(ValveState.Off)
                stdscr.addstr(4,0,"Standby",curses.A_BOLD)

        # Parse the next character from the keyboard
        if not vs.parse(vs,stdscr.getch()):
            if arg4 == "upgrade":

                # Perform the upgrade
                if not os.path.isdir("/tmp/upgrade"):
                    os.mkdir("/tmp/upgrade")

                # Copy the upgrade file onto the disk
                os.system("cp /media/disk/" + g_lEnumeratedNames[vs.value] + " /tmp/upgrade/")

                # Files have been copied so unmount the drive
                os.system("umount /media/disk")

                g_lEnumeratedNames[0] = g_lEnumeratedNames[vs.value]

            if g_eValveState != ValveState.Unknown:
                g_dValveState.set(ValveState.Off)
            coredata.bool("UpdateMenu").set(True)
            break;

# Main Function
if __name__ =='__main__':
    # in case of execution error, have a smooth recovery and clear
    # display of error message
    try:
        if sys.argv[4] == "list":
            g_lEnumeratedNames = sys.argv[5].split(",")

        elif sys.argv[4] == "upgrade":
            if not os.path.isdir("/media/disk"):
                os.mkdir("/media/disk")

            try:
                # make sure the USB device is there
                assert os.path.exists("/dev/usbdisk")
                os.system("mount /dev/usbdisk /media/disk")
                for file in os.listdir("/media/disk"):
                    if fnmatch.fnmatch(file,'K*_6100_Version_*_*_*.upgrade'):
                        g_lEnumeratedNames.append(file)
                # make sure there are some files to upgrade with
                assert len(g_lEnumeratedNames) != 0

            # There was an error trying to upgrade the system
            except AssertionError:
                # Try to unmount the drive
                os.system("umount /media/disk")

                # Communicate the error
                coredata.bool(sys.argv[3]).set(False)

                # Reraise the exception so it is caught
                raise

        if sys.argv[3] == "Config_AugerFwdMaximumCurrent_mA":
            g_dValveState = coredata.int32("AugerControlState")
            g_eValveState = ValveState.ForwardMax
        elif sys.argv[3] == "Config_AugerFwdMinimumCurrent_mA":
            g_dValveState = coredata.int32("AugerControlState")
            g_eValveState = ValveState.ForwardMin
        elif sys.argv[3] == "Config_AugerRevMaximumCurrent_mA":
            g_dValveState = coredata.int32("AugerControlState")
            g_eValveState = ValveState.ReverseMax
        elif sys.argv[3] == "Config_AugerRevMinimumCurrent_mA":
            g_dValveState = coredata.int32("AugerControlState")
            g_eValveState = ValveState.ReverseMin
        elif sys.argv[3] == "Config_SpinnerFwdMaximumCurrent_mA":
            g_dValveState = coredata.int32("SpinnerControlState")
            g_eValveState = ValveState.ForwardMax
        elif sys.argv[3] == "Config_SpinnerFwdMinimumCurrent_mA":
            g_dValveState = coredata.int32("SpinnerControlState")
            g_eValveState = ValveState.ForwardMin
        elif sys.argv[3] == "Config_SpinnerRevMaximumCurrent_mA":
            g_dValveState = coredata.int32("SpinnerControlState")
            g_eValveState = ValveState.ReverseMax
        elif sys.argv[3] == "Config_SpinnerRevMinimumCurrent_mA":
            g_dValveState = coredata.int32("SpinnerControlState")
            g_eValveState = ValveState.ReverseMin
        elif sys.argv[3] == "Config_PrewetMaximumCurrent_mA":
            g_dValveState = coredata.int32("PrewetControlState")
            g_eValveState = ValveState.ForwardMax
        elif sys.argv[3] == "Config_PrewetMinimumCurrent_mA":
            g_dValveState = coredata.int32("PrewetControlState")
            g_eValveState = ValveState.ForwardMin

        # Use the wrapper library to perform all the necessary terminal setup
        curses.wrapper(main,sys.argv[1],sys.argv[2],sys.argv[3],sys.argv[4])

        if sys.argv[4] == "upgrade":
            os.system("ratpoison -c 'vsplit 480/600'")
            os.system("tar -xvjpf /tmp/upgrade/" + g_lEnumeratedNames[0] + " -C /tmp/upgrade/")
            os.system("sh /tmp/upgrade/upgrade.sh")

    except:
        # print error message re exception
        traceback.print_exc()

        # Added to prevent the program from exiting when an error occurs.
        input()
