-- FILE:         createdb.sql
-- COPYRIGHT:    (c) 2009 Force America, Inc
-- DESCRIPTION:  SQL file that contains DDL statements for creating the
--               snapshot event database.
-- CREATED:      6/7/2009 By Jon Polfer (P1131)
-- MODIFIED:     By Jon Polfer (P1131)
--               -Modified to reflect rev7 of Snapshot Data Schema Design.
--               6/25/2010 By Jon Polfer (P1131)
--               -Modified to reflect rev8 of Snapshot Data Schema Design.

BEGIN TRANSACTION;

CREATE TABLE snapshot
(
    snapshot_id              INTEGER PRIMARY KEY AUTOINCREMENT,
    driver                   TEXT,
    snapshot_type            INTEGER,  
        -- defined by SnapshotType enum in libcoredata
    last_date_cleared        TEXT,
    date                     TEXT,
    -- data that has the scope of a single snapshot 
    -- Note:  inactive_* columns are deprecated
    inactive_distance_km     REAL,
    inactive_time_hms        TEXT,
    system_distance_km       REAL,
    system_time_hms          TEXT,
    granular_unload_time_hms TEXT,
    prewet_unload_time_hms   TEXT,
    direct_unload_time_hms   TEXT,
    CONSTRAINT snapshot_unique 
        UNIQUE (driver, snapshot_type, date) 
        ON CONFLICT ROLLBACK
);    

CREATE TABLE granular_material
(
    snapshot_id             INTEGER REFERENCES snapshot(snapshot_id),
    name                    TEXT,
    -- data totals to store 
    dry_distance_km         REAL,
    dry_weight_kg           REAL,
    dry_time_closed_hms     TEXT,
    dry_time_open_hms       TEXT,
    dry_time_manual_hms     TEXT,
    dry_time_blast_hms      TEXT,
    CONSTRAINT granular_pk 
        PRIMARY KEY (snapshot_id, name) 
        ON CONFLICT ROLLBACK
);

CREATE TABLE direct_liquid_material
(
    snapshot_id            INTEGER REFERENCES snapshot(snapshot_id),
    name                   TEXT,
    -- data totals to store 
    direct_distance_km     REAL,
    direct_volume_l        REAL,
    direct_time_closed_hms TEXT,
    direct_time_open_hms   TEXT,
    direct_time_manual_hms TEXT,
    direct_time_blast_hms  TEXT,
    CONSTRAINT direct_liquid_pk 
        PRIMARY KEY (snapshot_id, name) 
        ON CONFLICT ROLLBACK
);

CREATE TABLE prewetted_granular
(
    snapshot_id                     INTEGER REFERENCES snapshot(snapshot_id),
    granular_material_name          TEXT,   -- contains same values as granular_material.name
    prewet_material_name            TEXT,   -- contains same values as prewetted_simultaneous_granular_direct.prewet_material_name
    -- data totals to store
    -- Combined usage
    prewetted_distance_km           REAL,
    -- Prewetted granular usage
    prewetted_material_weight_kg    REAL,
    prewetted_time_open_hms         TEXT,
    prewetted_time_closed_hms       TEXT,
    prewetted_time_manual_hms       TEXT,
    prewetted_time_blast_hms        TEXT,
    -- actual prewet material usage
    prewet_material_volume_l        REAL,
    prewet_material_time_closed_hms TEXT,
    prewet_material_time_open_hms   TEXT,
    prewet_material_time_manual_hms TEXT,
    CONSTRAINT prewetted_granular_pk 
        PRIMARY KEY (snapshot_id, prewet_material_name, granular_material_name) 
        ON CONFLICT ROLLBACK
);

CREATE TABLE simultaneous_granular_direct
(
    snapshot_id                              INTEGER REFERENCES snapshot(snapshot_id),
    granular_material_name                   TEXT,   -- contains same values as granular_material.name
    direct_material_name                     TEXT,   -- contains same values as direct_liquid_material.name
    -- data totals to store
    -- Combined usage
    simultaneous_distance_km                 REAL,
    simultaneous_time_blast_hms              TEXT,
    -- Direct usage
    simultaneous_direct_volume_l             REAL,
    simultaneous_direct_time_closed_hms      TEXT,
    simultaneous_direct_time_open_hms        TEXT,
    simultaneous_direct_time_manual_hms      TEXT,
    -- Granular usage
    simultaneous_granular_weight_kg          REAL,
    simultaneous_granular_time_closed_hms    TEXT,
    simultaneous_granular_time_open_hms      TEXT,
    simultaneous_granular_time_manual_hms    TEXT,
    CONSTRAINT simultaneous_granular_direct_pk 
        PRIMARY KEY (snapshot_id, granular_material_name, direct_material_name) 
        ON CONFLICT ROLLBACK
);

CREATE TABLE prewetted_simultaneous_granular_direct
(
    snapshot_id                              INTEGER REFERENCES snapshot(snapshot_id),
    granular_material_name                   TEXT,    -- contains same values as granular_material.name
    direct_material_name                     TEXT,    -- contains same values as direct_liquid_material.name
    prewet_material_name                     TEXT,    -- contains same values as prewetted_granular.prewet_material_name
    -- data totals to store
    -- Combined usage
    simultaneous_distance_km                 REAL,
    simultaneous_time_blast_hms              TEXT,
    -- Direct usage
    simultaneous_direct_volume_l             REAL,
    simultaneous_direct_time_closed_hms      TEXT,
    simultaneous_direct_time_open_hms        TEXT,
    simultaneous_direct_time_manual_hms      TEXT,
    -- Prewetted granular usage
    prewetted_material_weight_kg             REAL,
    prewetted_time_open_hms                  TEXT,
    prewetted_time_closed_hms                TEXT,
    prewetted_time_manual_hms                TEXT,
    -- actual prewet material usage
    prewet_material_volume_l                 REAL,
    prewet_material_time_closed_hms          TEXT,
    prewet_material_time_open_hms            TEXT,
    prewet_material_time_manual_hms          TEXT,
    CONSTRAINT prewetted_simultaneous_granular_direct_pk 
        PRIMARY KEY (snapshot_id, prewet_material_name, granular_material_name, direct_material_name) 
        ON CONFLICT ROLLBACK
);

CREATE TABLE meta 
(
    snapshot_schema_version    INTEGER
);

INSERT INTO meta(snapshot_schema_version) VALUES (8);

COMMIT TRANSACTION;
