#!/bin/sh

################################################################################
#  FILE:         rev8torev9dbupgrade.sh
#  COPYRIGHT:    (c) 2011 FORCE America, Inc
#  DESCRIPTION:  Upgrades a rev9 database to a rev10 database.
#  CREATED:      08/11/2011 By Dmitry Lapan (P1131)
#  PARAMETERS:   $1 - path to database to upgrade
################################################################################

################################################################################
# FUNCTION:     is_integer
# DESCRIPTION:  Returns whether/not a variable contains an integer.
# CREATED:      12/27/2010 By Jon Polfer (P1131)
# PARAMETER:    $1 - variable to check 
# RETURNS:      0 if integer; 1 if not
################################################################################
function is_integer
{
    if [[ $(echo $1 | grep -E "^[0-9]+$") ]]; then
        return 0;
    else
        return 1;
    fi
}

################################################################################
# FUNCTION:     main
# DESCRIPTION:  Main thread of execution
# CREATED:      12/27/2010 By Jon Polfer (P1131)
# MODIFIED:     08/11/2011 By Dmitry Lapan (P1131)
#               -- Added the copying of e73xt_mixed_material data.
################################################################################

# Step 1: Check that database isn't >= rev10.
DbRevision=`echo "SELECT snapshot_schema_version FROM meta;" | sqlite3 $1`;
is_integer $DbRevision;
IsIntegerResult=$?;
if [[ $IsIntegerResult -eq 0 ]]; then
    if [[ "$DbRevision" -ge 10 ]]; then
        exit 0;
    fi
fi

# Step 2: Back up current database
cp $1 $1.old;

# Step 3: Delete current database
rm $1;

# Step 4: Create new empty database with rev10 database schema
sqlite3 $1 < createdb_rev10.sql;

# Step 5: Leave if old database contained no snapshots
NumberOfSnapshots=`echo "SELECT count(*) from snapshot;" | sqlite3 $1.old`;
if [[ $NumberOfSnapshots -eq 0 ]]; then
        exit 0;
fi

# Step 6: Copy data from the backup database
sqlite3 $1 <<-COPY_DATABASE_SQL
    ATTACH '$1.old' as source;

    -- copy snapshot table
    INSERT INTO snapshot(snapshot_id,
                         driver,
                         snapshot_type,
                         last_date_cleared,
                         date,
                         inactive_distance_km,
                         inactive_time_hms,
                         system_distance_km,
                         system_time_hms,
                         granular_unload_time_hms,
                         prewet_unload_time_hms,
                         direct_unload_time_hms,
                         e73xt_mixed_material_unload_time_hms)
    SELECT snapshot_id,
           driver,
           snapshot_type,
           last_date_cleared,
           date,
           inactive_distance_km,
           inactive_time_hms,
           system_distance_km,
           system_time_hms,
           granular_unload_time_hms,
           prewet_unload_time_hms,
           direct_unload_time_hms,
           e73xt_mixed_material_unload_time_hms
    FROM source.snapshot;
                         
    -- copy granular_material table
    INSERT INTO granular_material(snapshot_id,
                                  name,
                                  dry_distance_km,
                                  dry_weight_kg,
                                  dry_time_closed_hms,
                                  dry_time_open_hms,
                                  dry_time_manual_hms,
                                  dry_time_blast_hms)
    SELECT snapshot_id,
           name,
           dry_distance_km,
           dry_weight_kg,
           dry_time_closed_hms,
           dry_time_open_hms,
           dry_time_manual_hms,
           dry_time_blast_hms
    FROM source.granular_material;

    -- copy prewetted_granular table
    INSERT INTO prewetted_granular(snapshot_id,
                                   granular_material_name,
                                   prewet_material_name,
                                   prewetted_material_weight_kg,
                                   prewetted_distance_km,
                                   prewetted_time_open_hms,
                                   prewetted_time_closed_hms,
                                   prewetted_time_manual_hms,
                                   prewetted_time_blast_hms,
                                   prewet_material_volume_l,
                                   prewet_material_time_closed_hms,
                                   prewet_material_time_open_hms,
                                   prewet_material_time_manual_hms)
    SELECT snapshot_id,
           granular_material_name,
           prewet_material_name,
           prewetted_material_weight_kg,
           prewetted_distance_km,
           prewetted_time_open_hms,
           prewetted_time_closed_hms,
           prewetted_time_manual_hms,
           prewetted_time_blast_hms,
           prewet_material_volume_l,
           prewet_material_time_closed_hms,
           prewet_material_time_open_hms,
           prewet_material_time_manual_hms
    FROM source.prewetted_granular;

    -- copy direct_liquid_material table
    INSERT INTO direct_liquid_material(snapshot_id,
                                       name,
                                       direct_distance_km,
                                       direct_volume_l,
                                       direct_time_closed_hms,
                                       direct_time_open_hms,
                                       direct_time_manual_hms,
                                       direct_time_blast_hms)
    SELECT snapshot_id,
           name,
           direct_distance_km,
           direct_volume_l,
           direct_time_closed_hms,
           direct_time_open_hms,
           direct_time_manual_hms,
           direct_time_blast_hms
    FROM source.direct_liquid_material;

    -- copy simultaneous_granular_direct table
    INSERT INTO simultaneous_granular_direct(snapshot_id,
                                             granular_material_name,
                                             direct_material_name,
                                             simultaneous_distance_km,
                                             simultaneous_time_blast_hms,
                                             simultaneous_direct_volume_l,
                                             simultaneous_direct_time_closed_hms,
                                             simultaneous_direct_time_open_hms,
                                             simultaneous_direct_time_manual_hms,
                                             simultaneous_granular_weight_kg,
                                             simultaneous_granular_time_closed_hms,
                                             simultaneous_granular_time_open_hms,
                                             simultaneous_granular_time_manual_hms)
    SELECT    snapshot_id,
              granular_material_name,
              direct_material_name,
              simultaneous_distance_km,
              simultaneous_time_blast_hms,
              simultaneous_direct_volume_l,
              simultaneous_direct_time_closed_hms,
              simultaneous_direct_time_open_hms,
              simultaneous_direct_time_manual_hms,
              simultaneous_granular_weight_kg,
              simultaneous_granular_time_closed_hms,
              simultaneous_granular_time_open_hms,
              simultaneous_granular_time_manual_hms
    FROM source.simultaneous_granular_direct;
    

    -- copy prewetted_simultaneous_granular_direct table
    INSERT INTO prewetted_simultaneous_granular_direct(snapshot_id,
                                                       granular_material_name,
                                                       direct_material_name,
                                                       prewet_material_name,
                                                       simultaneous_distance_km,
                                                       simultaneous_time_blast_hms,
                                                       simultaneous_direct_volume_l,
                                                       simultaneous_direct_time_closed_hms,
                                                       simultaneous_direct_time_open_hms,
                                                       simultaneous_direct_time_manual_hms,
                                                       prewetted_material_weight_kg,
                                                       prewetted_time_open_hms,
                                                       prewetted_time_closed_hms,
                                                       prewetted_time_manual_hms,
                                                       prewet_material_volume_l,
                                                       prewet_material_time_closed_hms,
                                                       prewet_material_time_open_hms,
                                                       prewet_material_time_manual_hms)
    SELECT snapshot_id,
           granular_material_name,
           direct_material_name,
           prewet_material_name,
           simultaneous_distance_km,
           simultaneous_time_blast_hms,
           simultaneous_direct_volume_l,
           simultaneous_direct_time_closed_hms,
           simultaneous_direct_time_open_hms,
           simultaneous_direct_time_manual_hms,
           prewetted_material_weight_kg,
           prewetted_time_open_hms,
           prewetted_time_closed_hms,
           prewetted_time_manual_hms,
           prewet_material_volume_l,
           prewet_material_time_closed_hms,
           prewet_material_time_open_hms,
           prewet_material_time_manual_hms
    FROM source.prewetted_simultaneous_granular_direct;

    -- copy e73xt_mixed_material table
    INSERT INTO e73xt_mixed_material(snapshot_id,
                                     name,
                                     mixed_lane_distance_km,
                                     mixed_linear_distance_km,
                                     mixed_granular_weight_kg,
                                     mixed_liquid_volume_l,
                                     mixed_time_closed_hms,
                                     mixed_time_open_hms,
                                     mixed_time_manual_hms,
                                     mixed_time_blast_hms)
    SELECT snapshot_id,
           name,
           mixed_lane_distance_km,
           mixed_linear_distance_km,
           mixed_granular_weight_kg,
           mixed_liquid_volume_l,
           mixed_time_closed_hms,
           mixed_time_open_hms,
           mixed_time_manual_hms,
           mixed_time_blast_hms
    FROM source.e73xt_mixed_material;

    DETACH source;
COPY_DATABASE_SQL


