#!/bin/sh

####################################################################
#  File Name:    upgrade.sh
#  Description:  Upgrades a 6100 from an old firmware version to a new one.
#  Copyright:    (c) 2009 Force America, Inc.
#  Created:      10/10/2008 By Andrew Dorney and Patrick McElwee
#  Modified:     10/17/2008 By Andrew Dorney
#  Modified:	 01/09/2009 By Andrew Dorney
#                - Comment cleanup/retcon. :)
#                - Ensure that package upgrades don't stall if they try to ask
#                  a question about how to upgrade a config file - force the
#                  default behavior to happen instead.
# Modified:      01/20/2009 By Andrew Dorney
#                - Ensure that we move the 6100 splashy theme to its new location.
#                - Ensure that splashy gets set up correctly with the new kernel.
# Modified:      11/10/2009 By Andrew Dorney
#                - Cleanup for Gen2/Gen3 systems with Debian packaging.
# Modified:      12/2/2009 By Jon Polfer (P1131)
#                - Added code to backup/restore snapshot database
# Modified:      08/20/2010 By Andrew Dorney
#                - Force users upgrading from version =< 0.20 to use 
#                  a new upgrade file.
#                - Removed xml detection from 6100-svn postinst.
# Modified:      06/06/2011 By Andrew Dorney
#                - Make sure the 6100 ignores menu prompts and uses
#                  default package config settings. This is needed
#                  if we ever want to remote-install Samba, for ex.
####################################################################


# Global variables.
LOGFILE=/var/log/6100_software_update.log
MAIN_DIR=/tmp/upgrade
SNAPSHOT_UPGRADE_DIR=/tmp/upgrade/DataLogger/snapshot_upgrade
PREUPGRADE_VERSION="$(apt-cache show 6100-svn | grep Version: | cut -c 10-)"


####################################################################
# Name:          backup_snapshot_database
# Description:   Copies the snapshot_database to a temp folder
#                before the upgrade runs
# Created:       12/2/2009 By Jon Polfer (P1131)
# Parameters:    None
# Returns:       Nothing
####################################################################
function backup_snapshot_database 
{
    # Copy snapshot.db to /tmp/upgrade/backup_snapshot.db
    cp /6100/etc/snapshot.db /tmp/upgrade/backup_snapshot.db
}

####################################################################
# Name:          restore_snapshot_database
# Description:   Copies the backed-up database to the 6100 folder. 
# Created:       12/2/2009 By Jon Polfer (P1131)
# Parameters:    None
# Returns:       Nothing
####################################################################
function restore_snapshot_database 
{
    # Copy /tmp/upgrade/backup_snapshot.db to /6100/etc/snapshot.db
    cp /tmp/upgrade/backup_snapshot.db /6100/etc/snapshot.db
}

####################################################################
# Name:          restore_upgrade_snapshot_database
# Description:   Restores the snapshot database to the 6100 tree,
#                and runs all upgrade scripts on it 
# Created:       12/2/2009 By Jon Polfer (P1131)
# Parameters:    None
# Returns:       Nothing
# Modified:      8/16/2010 By Jon Polfer (P1131)
#                - Revised script to upgrade to rev 8
####################################################################
function restore_upgrade_snapshot_database 
{
    # Copy /tmp/upgrade/backup_snapshot.db to /6100/etc/snapshot.db
    cp /tmp/upgrade/backup_snapshot.db /6100/etc/snapshot.db
 
    # Move to the snapshot_upgrade directory
    cd $SNAPSHOT_UPGRADE_DIR

    # Upgrade the schema through its revisions in-order
    cat rev1torev5dbupgrade.sql | sqlite3 /6100/etc/snapshot.db
    cat cleanrev5data.sql | sqlite3 /6100/etc/snapshot.db
    cat rev5torev7dbupgrade.sql | sqlite3 /6100/etc/snapshot.db
    ./rev7torev8dbupgrade.sh /6100/etc/snapshot.db
    ./rev8torev9dbupgrade.sh /6100/etc/snapshot.db
    ./rev9torev10dbupgrade.sh /6100/etc/snapshot.db
    ./rev10torev11dbupgrade.sh /6100/etc/snapshot.db

    # Go back to the main directory
    cd $MAIN_DIR
}

####################################################################
#  Name:         cleanup_and_request_powerdown
#  Description:  Removes the /tmp/upgrade folder, cleans debian packages,
#                and informs the user of success or failure.
#                It then loops infinitely until the system is shut down.
#  Created:      By Andrew Dorney (P1131)
#  Parameters:   First parameter is a string, either "success" or "fail".
#                Parameter determines message sent to user.
####################################################################
function cleanup_and_request_powerdown {
	
	# Delete the /tmp/upgrade folder.
	# TODO - make the 6100 software do this.
	rm -rf /tmp/upgrade >> $LOGFILE 2>&1

	# If this function receives "success" as a parameter...
	if [[ "$1" = "success" ]]; then

		# Tell the logfile the upgrade was completed successfully.
		echo "Upgrade completed successfully!" >> $LOGFILE

		# Tell the user the upgrade was completed successfully.
		/6100/rc/pyclient set bool UpgradeSuccessful 1

		# Output a console message for old upgrades
		echo "Upgrade completed successfully!"
		echo "Shut off key power to the system."
	
	# If this function receives a "fail" as a parameter...
	elif [[ "$1" = "fail" ]]; then

		# Tell the logfile the upgrade was not completed successfully.
		echo "Upgrade was not completed successfully." >> $LOGFILE

		# Tell the user the upgrade was not completed successfully.
		/6100/rc/pyclient set bool UpgradeSuccessful ""

		# Output a console message for old upgrades
		echo "Upgrade failed!"
		echo "Shut off key power to the system."
	fi
	
	# Loop forever until the user shuts the system down.
	while true; do
		sleep 1
	done
}


####################################################################
#  Name:         xml_upgrade
#  Description:  Checks to see if the user needs a new XML file, and 
#                replaces it if necessary.
#  Created:      By Andrew Dorney (P1131)
#  Parameters:   None.
#  Modified:     3/15/2011 by Andrew Dorney (P1131)
#                 - Now checks for XML Rev 2 along with Rev 1.
####################################################################
function xml_upgrade
{
    # If the user is missing both a Rev 2 and a Rev 1 XML file,
	# or the user is upgrading from 6100-SVN less than or equal to 4170 
	# (the SVN rev where we forced the user to upgrade their XML file), 
	# give them the default.
	#
    # Otherwise, do nothing. This ensures that people with working Rev 2 and 
    # Rev 1 XML files keep theirs, as appropriate.
    if [[ ((! -e /6100/etc/6100_Config_2.xml) && (! -e /6100/etc/6100_Config.xml)) || ($PREUPGRADE_VERSION -le 4170) ]]; then
	    cp /6100/etc/6100_Config_2.defaults /6100/etc/6100_Config_2.xml
	fi
}


####################################################################
# Name:          main (well, not really, but you get the point)
# Description:   Point of entry for the upgrade script.
# Parameters:    None.
# Created:       11/10/2009 By Andrew Dorney
####################################################################

# Move to MAIN_DIR to ensure we can find the debian packages.
cd $MAIN_DIR

# Check for and remove all archived Debian packages, to save space.
LIST_OF_ARCHIVED_DEBIAN_PACKAGES=$(ls /var/cache/apt/archives/*.deb)
if [[ "$LIST_OF_ARCHIVED_DEBIAN_PACKAGES" != "" ]]; then
    rm /var/cache/apt/archives/*.deb
fi

# Check for and remove any WIFI Sensor Logs left around from releases 0.34 through 0.55.
#
#  find       /6100/SensorLogs/         ! -type d                  -name WIFIlog_*.csv          ! -name WIFIlog_*$(date +%m)*$(date +%Y).csv       -delete
# Find in the SensorLogs folder, where type is not directory, all files named WIFIlog_*.csv, except for those that match today's month and year. Then, delete them.
#
find /6100/SensorLogs/ ! -type d -name WIFIlog_*.csv ! -name WIFIlog_*$(date +%m)*$(date +%Y).csv -delete

# Backup the snapshot (standard logging) database
backup_snapshot_database

# Stop the persistd daemon from running.  Attempt to stop database corruption
/etc/init.d/FA_persistd stop

# Install any debian packages that aren't on the system.
#
# DEBIAN_FRONTEND=noninteractive ensures that if a package tries to pop 
# up a menu (like samba-common does), we use its defaults and carry on.
#
# This doesn't validate pkg signatures (--no-debsig), forces a new file 
# to overwrite an older config file (--force-confnew), and doesn't 
# install downgrades, only newer packages (-G). -E saves us time and 
# hassle by not reinstalling packages we already have. See "man dpkg".
DEBIAN_FRONTEND=noninteractive dpkg --install --no-debsig --force-confnew -G ./debs/*.deb

# If dpkg failed for any reason, log a failure and quit.
if [[ $? != 0 ]]; then
	restore_snapshot_database
	cleanup_and_request_powerdown fail
fi

# Check for XML file upgrade.
xml_upgrade

# Restore / Upgrade the snapshot database
restore_upgrade_snapshot_database

# Otherwise, we were sucessful!
cleanup_and_request_powerdown success

exit 0

